<?php

namespace Drupal\in\Form;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\in\InHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for IN module content that is displayed to users.
 *
 * @package Drupal\in\Form
 */
class InContentForm extends FormBase {

  use StringTranslationTrait;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The UUID generator.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuid;

  /**
   * Constructs a new InContentForm object.
   */
  public function __construct(
    StateInterface $state,
    MessengerInterface $messenger,
    UuidInterface $uuid
  ) {
    $this->state = $state;
    $this->messenger = $messenger;
    $this->uuid = $uuid;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('state'),
      $container->get('messenger'),
      $container->get('uuid')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'in_content';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $items = $this->state->get(InHelper::IN_STATE_CONF, []);
    if (count($items) > 0) {
      $form['settings'] = [
        '#type' => 'table',
        '#header' => [
          '',
          $this->t('Weight'),
          $this->t('Enabled'),
          $this->t('Type/Style'),
          $this->t('Text'),
          $this->t('Delete'),
        ],
        '#attributes' => [
          'id' => 'in-content-table',
        ],
        '#tabledrag' => [
          [
            'action' => 'order',
            'relationship' => 'sibling',
            'group' => 'draggable-weight',
          ],
        ],
      ];
    }
    else {
      $form['message'] = [
        '#type' => 'markup',
        '#markup' => '<p>' . t('There are no Important Notifications yet. Just add a new one.') . '</p>',
      ];
    }

    foreach ($items as $item) {
      $weight = $item['weight'];

      $form['settings'][$item['id']] = [
        'data' => [],
      ];
      $form['settings'][$item['id']]['#attributes']['class'] = ['draggable'];
      $form['settings'][$item['id']]['weight'] = [
        '#type' => 'weight',
        '#title' => t('Weight'),
        '#title_display' => 'invisible',
        '#default_value' => $weight,
        '#attributes' => [
          'class' => [
            'draggable-weight'
          ]
        ],
      ];
      $form['settings'][$item['id']]['enabled'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enabled'),
        '#title_display' => 'invisible',
        '#default_value' => $item['enabled'] ?? TRUE,
      ];
      $form['settings'][$item['id']]['type'] = [
        '#type' => 'select',
        '#title' => t('Type'),
        '#title_display' => 'invisible',
        '#default_value' => $item['type'] ?? 0,
        '#options' => $this->getTypeOptions(),
      ];
      $form['settings'][$item['id']]['text'] = [
        '#type' => 'text_format',
        '#default_value' => $item['text']['value'] ?? '',
        '#format' => $item['text']['format'],
        '#rows' => '3',
      ];
      $form['settings'][$item['id']]['delete'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Delete'),
        '#default_value' => FALSE,
      ];
    }

    $form['add'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add new notification'),
      '#attributes' => [
        'class' => [
          'button--action',
        ],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save configuration'),
      '#attributes' => [
        'class' => [
          'button button--primary',
        ],
      ],
      '#disabled' => count($items) === 0,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    if ($triggering_element['#id'] == "edit-submit") {
      $configs = [];
      $values = $form_state->getValue('settings', []);
      $deleted = 0;
      foreach ($values as $id => $value) {
        if (isset($value['delete']) && $value['delete'] == 1) {
          $deleted++;
        }
        else {
          // Prepare the config array.
          $configs[$id] = [
            'id' => $id,
            'enabled' => $value['enabled'] ?? TRUE,
            'type' => $value['type'] ?? 0,
            'text' => [
              'value' => $value['text']['value'],
              'format' => $value['text']['format'],
            ],
            'weight' => $value['weight'],
          ];
        }
      }

      if ($deleted > 0) {
        $this->messenger->addWarning(
          $this->formatPlural($deleted, '@count message was deleted.', '@count messages were deleted.')
        );
      }

      $this->state->set(InHelper::IN_STATE_CONF, $configs);
      $this->messenger->addStatus($this->t('Saved successfully.'));
    }
    else {
      $configs = $this->state->get(InHelper::IN_STATE_CONF, []);
      $new_id = $this->uuid->generate();
      $configs[$new_id] = [
        'id' => $new_id,
        'enabled' => TRUE,
        'type' => 0,
        'text' => [
          'value' => 'Hello World!',
          'format' => 'full_html',
        ],
        'weight' => 0,
      ];
      $this->state->set(InHelper::IN_STATE_CONF, $configs);
      $this->messenger->addStatus($this->t('Added new notification.'));
    }
  }

  /**
   * Returns the list of type for Important Notifications.
   *
   * @return array
   */
  private function getTypeOptions(): array {
    $list = [];
    $types = $this->state->get(InHelper::IN_STATE_TYPES, []);
    foreach ($types as $type) {
      $list[$type['machine_name']] = $type['label'];
    }
    return $list;
  }

}
